package onebit.chrdraw;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;

import javax.swing.JFrame;

import onebit.chrdraw.chr.Font;
import onebit.chrdraw.chr.FontFactory;
import onebit.chrdraw.chr.Glyph;


public class CHRDraw extends JFrame {

  CHRDraw(Font font, String text) {
    super("Borland .CHR font drawing test v0.3 (c) 2009 mono");
    setDefaultCloseOperation(EXIT_ON_CLOSE);
    setLayout(new BorderLayout(5, 5));
    
    TextLine picture = new TextLine(font, text);
    
    Container c = getContentPane();
    c.add(BorderLayout.CENTER, picture);
    
    setMinimumSize(new Dimension(100, 50));
  }
  
  static Font load(String filename) 
      throws FileNotFoundException, IOException {
    
    FileInputStream fis = new FileInputStream(filename);
    try {
      BufferedInputStream bis = new BufferedInputStream(fis);
      try {
        return FontFactory.load(bis);
        
      } finally {
        bis.close();
      }
    } finally {
      fis.close();
    }
  }
  
  static void help() {
    String text = "Borland .CHR font drawing test v0.3, (c) 2009 mono\n" +
        "Usage: java -jar chrdraw.jar <chr_file> [<example_text>]\n" +
        "Draw <example_text> on screen using <chr_file> font file\n" +
        "<example_text> is 'The quick brown fox jumps lazy dog.' by default.";
    System.out.println(text);
  }
  
  static void info(Font font) {
    String description = font.getDescription();
    Integer majorVersion = font.getDriverMajorVersion();
    Integer minorVersion = font.getDriverMinorVersion();
    String fontname = font.getFontname();
    String type = font.getFontType();
    String internal = font.getInternalFontname();
    Integer originToBaseLine = font.getOriginToBaseLine();
    Integer originToBottom = font.getOriginToBottomDescender();
    Integer originToTop = font.getOriginToTopOfCapital();
    Integer flag = font.getScanFlag();
    Character signature = font.getSignature();
    
    Character from = 0xff;
    Character to = 0;
    for (Glyph glyph : font.getCharacters().values()) {
      Character ch = glyph.getName();
      if (from > ch) {
        from = ch;
      }
      if (to < ch) {
        to = ch;
      }
    }
    
    System.out.println("Font type: " + type);
    System.out.println("Font name: " + fontname);
    System.out.println("Internal font name: " + internal);
    System.out.println("Description: " + description);
    System.out.println("Driver version: " + majorVersion + "." + minorVersion);
    System.out.println("Scan flag: " + flag);
    System.out.println("Signature: " + signature);
    System.out.println("Origin to baseline distance: " + originToBaseLine);
    System.out.println("Origin to bottom descender distance: " + originToBottom);
    System.out.println("Origin to top of capital distance: " + originToTop);
    
    System.out.println(String.format("Character range: %c(%02x)..%c(%02x)", from, (int)from, to, (int)to));
  }
  
  public static void main(String... args) {
    if (args.length > 0) {
      String chrFilename = null;
      String exampleText = "The quick brown fox jumps lazy dog.";

      Iterator<String> it = Arrays.asList(args).iterator();
      if (it.hasNext()) {
        chrFilename = it.next();
        if (it.hasNext()) {
          exampleText = it.next();
        }
      } else {
        throw new RuntimeException("No .CHR font file");
      }
      
      try {
        Font font = load(chrFilename);
        
        info(font);
        
        CHRDraw chrDraw = new CHRDraw(font, exampleText);
        chrDraw.pack();
        chrDraw.setVisible(true);
        
      } catch (IOException exception) {
        exception.printStackTrace();
      }
    } else {
      help();
    }
  }
}
